#include <SoftwareSerial.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <PubSubClient.h>
#include <EEPROM.h>
#include "WEBsetMQTT.html.h"

extern "C" {
#include "user_interface.h"
}
#define LEDpin D4        // LED zum Blinken. Bei ESP-07 Pin 2 D4. Bei ESP-01 Pin 1
#define DNS
#define debug true
IPAddress apIP(192, 168, 168, 30);

#if defined DNS
#include <DNSServer.h>
const byte DNS_PORT = 53;
DNSServer dnsServer;
#endif

char ssid[32] = "fhgdhfghd\0";
char passwort[64] = "gghdfgh\0";
String konfig[9];

ESP8266WebServer server;
EspClass esp;
int z = 0;                   //Aktuelle EEPROM-Adresse zum lesen
#include "Setup2.h"
int Messinterval = 10;

WiFiClient wclient;
PubSubClient client(wclient);

SoftwareSerial co2Serial(D2, D1); // RX, TX

int leseCO2()
{
  byte cmd[9] = {0xFF, 0x01, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x79};
  char antwort[9];
  co2Serial.write(cmd, 9);
  co2Serial.readBytes(antwort, 9);
  if (antwort[0] != 0xFF) return -1;
  if (antwort[1] != 0x86) return -1;
  int antwortHigh = (int) antwort[2];
  int antwortLow = (int) antwort[3];
  int ppm = (256 * antwortHigh) + antwortLow;
  return ppm;
}

void setup()
{
  char inser;               // Serielle daten ablegen
  EEPROM.begin(512);                                 // EEPROM initialisieren mit 200 Byts

  //------------------------------------ Pruefen ob Tx und Rx verbunden sind --------------
  Serial.begin(115200);
  while (Serial.available())inser = Serial.read();
  Serial.println("Start");
  delay(10);
  inser = Serial.read();
  if (inser != 'S') goto weiter;
  inser = Serial.read();
  if (inser != 't') goto weiter;
  pinMode(LEDpin, OUTPUT);
  digitalWrite(LEDpin, 0);
  Einstellen();

weiter:
  z = 0;
  LeseEeprom(ssid, sizeof(ssid));        // EEPROM lesen
  if (ssid[0] > 127) // Wenn EEPROM leer ist dann:
  {
    for (int i = 0; i < 20; i++) EEPROM.write(i, '\0');
    EEPROM.commit();
    pinMode(LEDpin, OUTPUT);
    digitalWrite(LEDpin, 0);
    Einstellen();
  }
  LeseEeprom(passwort, sizeof(passwort));
  for (int k = 0; k < 7; k++) LeseEepromStr(&konfig[k], 60);
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, passwort);
  int timout = 0;
  pinMode(LEDpin, OUTPUT);
  while (WiFi.status() != WL_CONNECTED)
  {
    timout++;
    if  (timout > 60) // Wenn Anmeldung nicht m�glich
    {
      digitalWrite(LEDpin, 0);    // LED ON f�r 5 Sekunden
      delay(5000);
      digitalWrite(LEDpin, 1);
      esp.deepSleep(Messinterval * 60e6, WAKE_RFCAL);
      delay(100);
    }        // End timeout
    delay(300);
    digitalWrite(LEDpin, 0);
    delay(300);
    digitalWrite(LEDpin, 1);
  }                         // Verbunden

  Serial.begin(115200);
  Serial.println("");
  Serial.println("Start");
  pinMode(2, OUTPUT);
  digitalWrite(2, 0);
  co2Serial.begin(9600);
}

void loop()
{
  if (WiFi.status() == WL_CONNECTED)
  {
    if (!client.connected()) //------------------------- MQTT nicht verbunden ----------------------------------------
    {
      client.set_server(konfig[0], konfig[1].toInt());
      if (client.connect(MQTT::Connect(konfig[4]).set_auth(konfig[2], konfig[3])))
      {
        String Antwort = String(leseCO2());
        client.publish(MQTT::Publish(konfig[5], Antwort).set_qos(0));
        client.loop();
        MQTT::Disconnect();
        Messinterval = konfig[6].toInt();
        esp.deepSleep(Messinterval * 60e6, WAKE_RFCAL);
      }
    } //----------------------------------------------- ende MQTT nicht verbunden -------------------------------------
    else
    {
      client.loop();
    }
  } // ende mit WLAN verbunden
  yield();

}
